@@ -1,7 +1,7 @@
 /*
  * wee-hook-process.c - WeeChat process hook
  *
- * Copyright (C) 2003-2023 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2024 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -146,6 +146,7 @@ hook_process_hashtable (struct t_weechat_plugin *plugin,
         ptr_value = hashtable_get (options, "buffer_flush");
         if (ptr_value && ptr_value[0])
         {
+            error = NULL;
             number = strtol (ptr_value, &error, 10);
             if (error && !error[0]
                 && (number >= 1) && (number <= HOOK_PROCESS_BUFFER_SIZE))
@@ -862,7 +863,7 @@ hook_process_free_data (struct t_hook *hook)
     if (HOOK_PROCESS(hook, child_pid) > 0)
     {
         kill (HOOK_PROCESS(hook, child_pid), SIGKILL);
-        hook_schedule_clean_children ();
+        hook_schedule_clean_process (HOOK_PROCESS(hook, child_pid));
         HOOK_PROCESS(hook, child_pid) = 0;
     }
     if (HOOK_PROCESS(hook, child_read[HOOK_PROCESS_STDIN]) != -1)
