@@ -1,7 +1,7 @@
 /*
  * wee-infolist.c - info lists management
  *
- * Copyright (C) 2003-2015 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2024 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -16,7 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -57,7 +57,7 @@ infolist_new (struct t_weechat_plugin *plugin)
 
         new_infolist->prev_infolist = last_weechat_infolist;
         new_infolist->next_infolist = NULL;
-        if (weechat_infolists)
+        if (last_weechat_infolist)
             last_weechat_infolist->next_infolist = new_infolist;
         else
             weechat_infolists = new_infolist;
@@ -114,7 +114,7 @@ infolist_new_item (struct t_infolist *infolist)
 
         new_item->prev_item = infolist->last_item;
         new_item->next_item = NULL;
-        if (infolist->items)
+        if (infolist->last_item)
             infolist->last_item->next_item = new_item;
         else
             infolist->items = new_item;
@@ -147,10 +147,11 @@ infolist_new_var_integer (struct t_infolist_item *item,
         new_var->value = malloc (sizeof (int));
         if (new_var->value)
             *((int *)new_var->value) = value;
+        new_var->size = 0;  /* not used for an integer */
 
         new_var->prev_var = item->last_var;
         new_var->next_var = NULL;
-        if (item->vars)
+        if (item->last_var)
             item->last_var->next_var = new_var;
         else
             item->vars = new_var;
@@ -181,10 +182,11 @@ infolist_new_var_string (struct t_infolist_item *item,
         new_var->name = strdup (name);
         new_var->type = INFOLIST_STRING;
         new_var->value = (value) ? strdup (value) : NULL;
+        new_var->size = 0;  /* not used for a string */
 
         new_var->prev_var = item->last_var;
         new_var->next_var = NULL;
-        if (item->vars)
+        if (item->last_var)
             item->last_var->next_var = new_var;
         else
             item->vars = new_var;
@@ -215,10 +217,11 @@ infolist_new_var_pointer (struct t_infolist_item *item,
         new_var->name = strdup (name);
         new_var->type = INFOLIST_POINTER;
         new_var->value = pointer;
+        new_var->size = 0;  /* not used for a pointer */
 
         new_var->prev_var = item->last_var;
         new_var->next_var = NULL;
-        if (item->vars)
+        if (item->last_var)
             item->last_var->next_var = new_var;
         else
             item->vars = new_var;
@@ -255,7 +258,7 @@ infolist_new_var_buffer (struct t_infolist_item *item,
 
         new_var->prev_var = item->last_var;
         new_var->next_var = NULL;
-        if (item->vars)
+        if (item->last_var)
             item->last_var->next_var = new_var;
         else
             item->vars = new_var;
@@ -288,10 +291,11 @@ infolist_new_var_time (struct t_infolist_item *item,
         new_var->value = malloc (sizeof (time_t));
         if (new_var->value)
             *((time_t *)new_var->value) = time;
+        new_var->size = 0;  /* not used for a time */
 
         new_var->prev_var = item->last_var;
         new_var->next_var = NULL;
-        if (item->vars)
+        if (item->last_var)
             item->last_var->next_var = new_var;
         else
             item->vars = new_var;
@@ -310,6 +314,9 @@ infolist_new_var_time (struct t_infolist_item *item,
 struct t_infolist_item *
 infolist_next (struct t_infolist *infolist)
 {
+    if (!infolist)
+        return NULL;
+
     if (!infolist->ptr_item)
     {
         infolist->ptr_item = infolist->items;
@@ -328,6 +335,9 @@ infolist_next (struct t_infolist *infolist)
 struct t_infolist_item *
 infolist_prev (struct t_infolist *infolist)
 {
+    if (!infolist)
+        return NULL;
+
     if (!infolist->ptr_item)
     {
         infolist->ptr_item = infolist->last_item;
@@ -344,6 +354,9 @@ infolist_prev (struct t_infolist *infolist)
 void
 infolist_reset_item_cursor (struct t_infolist *infolist)
 {
+    if (!infolist)
+        return;
+
     infolist->ptr_item = NULL;
 }
 
@@ -362,7 +375,7 @@ infolist_search_var (struct t_infolist *infolist, const char *name)
     for (ptr_var = infolist->ptr_item->vars; ptr_var;
          ptr_var = ptr_var->next_var)
     {
-        if (string_strcasecmp (ptr_var->name, name) == 0)
+        if (strcmp (ptr_var->name, name) == 0)
             return ptr_var;
     }
 
@@ -443,7 +456,7 @@ infolist_integer (struct t_infolist *infolist, const char *var)
     for (ptr_var = infolist->ptr_item->vars; ptr_var;
          ptr_var = ptr_var->next_var)
     {
-        if (string_strcasecmp (ptr_var->name, var) == 0)
+        if (strcmp (ptr_var->name, var) == 0)
         {
             if (ptr_var->type == INFOLIST_INTEGER)
                 return *((int *)ptr_var->value);
@@ -471,7 +484,7 @@ infolist_string (struct t_infolist *infolist, const char *var)
     for (ptr_var = infolist->ptr_item->vars; ptr_var;
          ptr_var = ptr_var->next_var)
     {
-        if (string_strcasecmp (ptr_var->name, var) == 0)
+        if (strcmp (ptr_var->name, var) == 0)
         {
             if (ptr_var->type == INFOLIST_STRING)
                 return (char *)ptr_var->value;
@@ -499,7 +512,7 @@ infolist_pointer (struct t_infolist *infolist, const char *var)
     for (ptr_var = infolist->ptr_item->vars; ptr_var;
          ptr_var = ptr_var->next_var)
     {
-        if (string_strcasecmp (ptr_var->name, var) == 0)
+        if (strcmp (ptr_var->name, var) == 0)
         {
             if (ptr_var->type == INFOLIST_POINTER)
                 return ptr_var->value;
@@ -530,7 +543,7 @@ infolist_buffer (struct t_infolist *infolist, const char *var,
     for (ptr_var = infolist->ptr_item->vars; ptr_var;
          ptr_var = ptr_var->next_var)
     {
-        if (string_strcasecmp (ptr_var->name, var) == 0)
+        if (strcmp (ptr_var->name, var) == 0)
         {
             if (ptr_var->type == INFOLIST_BUFFER)
             {
@@ -561,7 +574,7 @@ infolist_time (struct t_infolist *infolist, const char *var)
     for (ptr_var = infolist->ptr_item->vars; ptr_var;
          ptr_var = ptr_var->next_var)
     {
-        if (string_strcasecmp (ptr_var->name, var) == 0)
+        if (strcmp (ptr_var->name, var) == 0)
         {
             if (ptr_var->type == INFOLIST_TIME)
                 return *((time_t *)ptr_var->value);
@@ -584,6 +597,9 @@ infolist_var_free (struct t_infolist_item *item,
 {
     struct t_infolist_var *new_vars;
 
+    if (!item || !var)
+        return;
+
     /* remove var */
     if (item->last_var == var)
         item->last_var = var->prev_var;
@@ -625,6 +641,9 @@ infolist_item_free (struct t_infolist *infolist,
 {
     struct t_infolist_item *new_items;
 
+    if (!infolist || !item)
+        return;
+
     /* remove var */
     if (infolist->last_item == item)
         infolist->last_item = item->prev_item;
@@ -661,6 +680,9 @@ infolist_free (struct t_infolist *infolist)
 {
     struct t_infolist *new_weechat_infolists;
 
+    if (!infolist)
+        return;
+
     /* remove list */
     if (last_weechat_infolist == infolist)
         last_weechat_infolist = infolist->prev_infolist;
@@ -761,7 +783,7 @@ infolist_print_log ()
                         log_printf ("        size of buffer . . . : %d",    ptr_var->size);
                         break;
                     case INFOLIST_TIME:
-                        log_printf ("        value (time) . . . . : %ld", *((time_t *)ptr_var->value));
+                        log_printf ("        value (time) . . . . : %lld", (long long)(*((time_t *)ptr_var->value)));
                         break;
                 }
                 log_printf ("        prev_var . . . . . . : 0x%lx", ptr_var->prev_var);
