@@ -1,7 +1,7 @@
 /*
  * wee-network.c - network functions
  *
- * Copyright (C) 2003-2021 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2024 Sébastien Helleu <flashcode@flashtux.org>
  * Copyright (C) 2005-2010 Emmanuel Bouthenot <kolter@openics.org>
  * Copyright (C) 2010 Gu1ll4um3r0m41n <aeroxteam@gmail.com>
  * Copyright (C) 2012 Simon Arlott
@@ -68,6 +68,9 @@
 
 
 int network_init_gnutls_ok = 0;
+
+int network_num_certs_system = 0; /* number of system certs loaded          */
+int network_num_certs_user = 0;   /* number of user certs loaded            */
 int network_num_certs = 0;        /* number of certs loaded (system + user) */
 
 gnutls_certificate_credentials_t gnutls_xcred; /* GnuTLS client credentials */
@@ -88,6 +91,27 @@ network_init_gcrypt ()
     gcry_control (GCRYCTL_INITIALIZATION_FINISHED, 0);
 }
 
+/*
+ * Allocates credentials structure.
+ */
+
+void
+network_allocate_credentials ()
+{
+    gnutls_certificate_allocate_credentials (&gnutls_xcred);
+#if LIBGNUTLS_VERSION_NUMBER >= 0x02090a /* 2.9.10 */
+    gnutls_certificate_set_verify_function (gnutls_xcred,
+                                            &hook_connect_gnutls_verify_certificates);
+#endif /* LIBGNUTLS_VERSION_NUMBER >= 0x02090a */
+#if LIBGNUTLS_VERSION_NUMBER >= 0x020b00 /* 2.11.0 */
+    gnutls_certificate_set_retrieve_function (gnutls_xcred,
+                                              &hook_connect_gnutls_set_certificates);
+#else
+    gnutls_certificate_client_set_retrieve_function (gnutls_xcred,
+                                                     &hook_connect_gnutls_set_certificates);
+#endif /* LIBGNUTLS_VERSION_NUMBER >= 0x020b00 */
+}
+
 /*
  * Loads system's default trusted certificate authorities.
  *
@@ -125,6 +149,8 @@ network_load_system_ca_file (int force_display)
             rc);
     }
 
+    network_num_certs_system = rc;
+
     return rc;
 }
 
@@ -212,6 +238,8 @@ end:
     if (options)
         hashtable_free (options);
 
+    network_num_certs_user = num_loaded;
+
     return num_loaded;
 }
 
@@ -225,6 +253,8 @@ network_load_ca_files (int force_display)
     if (weechat_no_gnutls)
         return;
 
+    network_num_certs_system = 0;
+    network_num_certs_user = 0;
     network_num_certs = 0;
 
     network_num_certs += network_load_system_ca_file (force_display);
@@ -250,9 +280,7 @@ network_reload_ca_files (int force_display)
                              network_num_certs),
                          network_num_certs);
     }
-
-    gnutls_certificate_allocate_credentials (&gnutls_xcred);
-
+    network_allocate_credentials ();
     network_load_ca_files (force_display);
 }
 
@@ -266,19 +294,8 @@ network_init_gnutls ()
     if (!weechat_no_gnutls)
     {
         gnutls_global_init ();
-        gnutls_certificate_allocate_credentials (&gnutls_xcred);
+        network_allocate_credentials ();
         network_load_ca_files (0);
-#if LIBGNUTLS_VERSION_NUMBER >= 0x02090a /* 2.9.10 */
-        gnutls_certificate_set_verify_function (gnutls_xcred,
-                                                &hook_connect_gnutls_verify_certificates);
-#endif /* LIBGNUTLS_VERSION_NUMBER >= 0x02090a */
-#if LIBGNUTLS_VERSION_NUMBER >= 0x020b00 /* 2.11.0 */
-        gnutls_certificate_set_retrieve_function (gnutls_xcred,
-                                                  &hook_connect_gnutls_set_certificates);
-#else
-        gnutls_certificate_client_set_retrieve_function (gnutls_xcred,
-                                                         &hook_connect_gnutls_set_certificates);
-#endif /* LIBGNUTLS_VERSION_NUMBER >= 0x020b00 */
     }
 
     network_init_gnutls_ok = 1;
@@ -320,11 +337,11 @@ network_is_ip_address (const char *address)
         return 0;
 
     /* valid IPv4? */
-    if (inet_pton (AF_INET, address, &server_addr.sin_addr))
+    if (inet_pton (AF_INET, address, &server_addr.sin_addr) == 1)
         return 1;
 
     /* valid IPv6? */
-    if (inet_pton (AF_INET6, address, &server_addr6.sin6_addr))
+    if (inet_pton (AF_INET6, address, &server_addr6.sin6_addr) == 1)
         return 1;
 
     /* not a valid IP address */
@@ -537,11 +554,13 @@ network_pass_socks4proxy (struct t_proxy *proxy, int sock, const char *address,
     socks4.method = 1;
     socks4.port = htons (port);
     network_resolve (address, ip_addr, NULL);
-    socks4.address = inet_addr (ip_addr);
     strncpy (socks4.user, username, sizeof (socks4.user) - 1);
 
     free (username);
 
+    if (inet_pton (AF_INET, ip_addr, &socks4.address) != 1)
+        return 0;
+
     length = 8 + strlen (socks4.user) + 1;
     if (network_send_with_retry (sock, (char *) &socks4, length, 0) != length)
         return 0;
@@ -750,7 +769,7 @@ network_pass_proxy (const char *proxy, int sock, const char *address, int port)
     ptr_proxy = proxy_search (proxy);
     if (ptr_proxy)
     {
-        switch (CONFIG_INTEGER(ptr_proxy->options[PROXY_OPTION_TYPE]))
+        switch (CONFIG_ENUM(ptr_proxy->options[PROXY_OPTION_TYPE]))
         {
             case PROXY_TYPE_HTTP:
                 rc = network_pass_httpproxy (ptr_proxy, sock, address, port);
@@ -1809,7 +1828,7 @@ network_connect_with_fork (struct t_hook *hook_connect)
     const char *pos_error;
     pid_t pid;
 
-    /* initialize GnuTLS if SSL asked */
+    /* initialize GnuTLS if TLS asked */
     if (HOOK_CONNECT(hook_connect, gnutls_sess))
     {
         if (gnutls_init (HOOK_CONNECT(hook_connect, gnutls_sess), GNUTLS_CLIENT) != GNUTLS_E_SUCCESS)
