@@ -1,7 +1,7 @@
 /*
  * wee-upgrade-file.c - save/restore data for upgrading WeeChat
  *
- * Copyright (C) 2003-2015 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2024 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -16,7 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -184,7 +184,14 @@ upgrade_file_write_buffer (struct t_upgrade_file *upgrade_file, void *pointer,
  */
 
 struct t_upgrade_file *
-upgrade_file_new (const char *filename, int write)
+upgrade_file_new (const char *filename,
+                  int (*callback_read)(const void *pointer,
+                                       void *data,
+                                       struct t_upgrade_file *upgrade_file,
+                                       int object_id,
+                                       struct t_infolist *infolist),
+                  const void *callback_read_pointer,
+                  void *callback_read_data)
 {
     int length;
     struct t_upgrade_file *new_upgrade_file;
@@ -196,7 +203,7 @@ upgrade_file_new (const char *filename, int write)
     if (new_upgrade_file)
     {
         /* build name of file */
-        length = strlen (weechat_home) + 1 + strlen (filename) + 16 + 1;
+        length = strlen (weechat_data_dir) + 1 + strlen (filename) + 16 + 1;
         new_upgrade_file->filename = malloc (length);
         if (!new_upgrade_file->filename)
         {
@@ -204,13 +211,16 @@ upgrade_file_new (const char *filename, int write)
             return NULL;
         }
         snprintf (new_upgrade_file->filename, length, "%s/%s.upgrade",
-                  weechat_home, filename);
+                  weechat_data_dir, filename);
+        new_upgrade_file->callback_read = callback_read;
+        new_upgrade_file->callback_read_pointer = callback_read_pointer;
+        new_upgrade_file->callback_read_data = callback_read_data;
 
         /* open file in read or write mode */
-        if (write)
-            new_upgrade_file->file = fopen (new_upgrade_file->filename, "wb");
-        else
+        if (callback_read)
             new_upgrade_file->file = fopen (new_upgrade_file->filename, "rb");
+        else
+            new_upgrade_file->file = fopen (new_upgrade_file->filename, "wb");
 
         if (!new_upgrade_file->file)
         {
@@ -220,7 +230,7 @@ upgrade_file_new (const char *filename, int write)
         }
 
         /* change permissions if write mode */
-        if (write)
+        if (!callback_read)
         {
             chmod (new_upgrade_file->filename, 0600);
 
@@ -235,12 +245,13 @@ upgrade_file_new (const char *filename, int write)
         /* add upgrade file to list of upgrade files */
         new_upgrade_file->prev_upgrade = last_upgrade_file;
         new_upgrade_file->next_upgrade = NULL;
-        if (upgrade_files)
+        if (last_upgrade_file)
             last_upgrade_file->next_upgrade = new_upgrade_file;
         else
             upgrade_files = new_upgrade_file;
         last_upgrade_file = new_upgrade_file;
     }
+
     return new_upgrade_file;
 }
 
@@ -280,7 +291,11 @@ upgrade_file_write_object (struct t_upgrade_file *upgrade_file, int object_id,
         fields = infolist_fields (infolist);
         if (fields)
         {
-            argv = string_split (fields, ",", 0, 0, &argc);
+            argv = string_split (fields, ",", NULL,
+                                 WEECHAT_STRING_SPLIT_STRIP_LEFT
+                                 | WEECHAT_STRING_SPLIT_STRIP_RIGHT
+                                 | WEECHAT_STRING_SPLIT_COLLAPSE_SEPS,
+                                 0, &argc);
             if (argv && (argc > 0))
             {
                 for (i = 0; i < argc; i++)
@@ -477,27 +492,6 @@ upgrade_file_read_string (struct t_upgrade_file *upgrade_file, char **string)
     return 1;
 }
 
-/*
- * Reads an UTF-8 string in upgrade file (reads a string then normalizes it).
- *
- * Returns:
- *   1: OK
- *   0: error
- */
-
-int
-upgrade_file_read_string_utf8 (struct t_upgrade_file *upgrade_file,
-                               char **string)
-{
-    if (!upgrade_file_read_string (upgrade_file, string))
-        return 0;
-
-    if (string && *string)
-        utf8_normalize (*string, '?');
-
-    return 1;
-}
-
 /*
  * Reads a buffer in upgrade file.
  *
@@ -704,11 +698,15 @@ upgrade_file_read_object (struct t_upgrade_file *upgrade_file)
 
     if (upgrade_file->callback_read)
     {
-        if ((int)(upgrade_file->callback_read) (upgrade_file->callback_read_data,
-                                                upgrade_file,
-                                                object_id,
-                                                infolist) == WEECHAT_RC_ERROR)
+        if ((int)(upgrade_file->callback_read) (
+                upgrade_file->callback_read_pointer,
+                upgrade_file->callback_read_data,
+                upgrade_file,
+                object_id,
+                infolist) == WEECHAT_RC_ERROR)
+        {
             rc = 0;
+        }
     }
 
 end:
@@ -733,21 +731,13 @@ end:
  */
 
 int
-upgrade_file_read (struct t_upgrade_file *upgrade_file,
-                   int (*callback_read)(void *data,
-                                        struct t_upgrade_file *upgrade_file,
-                                        int object_id,
-                                        struct t_infolist *infolist),
-                   void *callback_read_data)
+upgrade_file_read (struct t_upgrade_file *upgrade_file)
 {
     char *signature;
 
-    if (!upgrade_file)
+    if (!upgrade_file || !upgrade_file->callback_read)
         return 0;
 
-    upgrade_file->callback_read = callback_read;
-    upgrade_file->callback_read_data = callback_read_data;
-
     signature = NULL;
     if (!upgrade_file_read_string (upgrade_file, &signature))
     {
@@ -776,12 +766,31 @@ upgrade_file_read (struct t_upgrade_file *upgrade_file,
 }
 
 /*
- * Closes an upgrade file.
+ * Closes and frees an upgrade file.
  */
 
 void
 upgrade_file_close (struct t_upgrade_file *upgrade_file)
 {
-    if (upgrade_file && upgrade_file->file)
+    if (!upgrade_file)
+        return;
+
+    if (upgrade_file->filename)
+        free (upgrade_file->filename);
+    if (upgrade_file->file)
         fclose (upgrade_file->file);
+    if (upgrade_file->callback_read_data)
+        free (upgrade_file->callback_read_data);
+
+    /* remove upgrade file list */
+    if (upgrade_file->prev_upgrade)
+        (upgrade_file->prev_upgrade)->next_upgrade = upgrade_file->next_upgrade;
+    if (upgrade_file->next_upgrade)
+        (upgrade_file->next_upgrade)->prev_upgrade = upgrade_file->prev_upgrade;
+    if (upgrade_files == upgrade_file)
+        upgrade_files = upgrade_file->next_upgrade;
+    if (last_upgrade_file == upgrade_file)
+        last_upgrade_file = upgrade_file->prev_upgrade;
+
+    free (upgrade_file);
 }
