@@ -1,7 +1,7 @@
 /*
  * gui-key.c - keyboard functions (used by all GUI)
  *
- * Copyright (C) 2003-2023 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2024 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -66,7 +66,7 @@ int gui_keys_count[GUI_KEY_NUM_CONTEXTS];            /* keys number         */
 int gui_default_keys_count[GUI_KEY_NUM_CONTEXTS];    /* default keys number */
 
 char *gui_key_context_string[GUI_KEY_NUM_CONTEXTS] =
-{ "default", "search", "cursor", "mouse" };
+{ "default", "search", "histsearch", "cursor", "mouse" };
 
 char *gui_key_focus_string[GUI_KEY_NUM_FOCUS] =
 { "*", "chat", "bar", "item" };
@@ -75,8 +75,8 @@ char *gui_key_modifier_list[] =
 { "meta-", "ctrl-", "shift-", NULL };
 
 char *gui_key_alias_list[] =
-{ "f0", "f1", "f2", "f3", "f4", "f5", "f6", "f7", "f8", "f9", "f10", "f11",
-  "f12", "f13", "f14", "f15", "f16", "f17", "f18", "f19", "f20",
+{ "f10", "f11", "f12", "f13", "f14", "f15", "f16", "f17", "f18", "f19", "f20",
+  "f0", "f1", "f2", "f3", "f4", "f5", "f6", "f7", "f8", "f9",
   "home", "insert", "delete", "end", "backspace", "pgup", "pgdn",
   "up", "down", "right", "left", "tab", "return", "comma", "space", NULL };
 
@@ -87,7 +87,6 @@ int gui_key_verbose = 0;            /* 1 to see some messages               */
 
 char gui_key_combo[1024];           /* buffer used for combos               */
 int gui_key_grab = 0;               /* 1 if grab mode enabled (alt-k)       */
-int gui_key_grab_raw = 0;           /* grab raw key code?                   */
 int gui_key_grab_count = 0;         /* number of keys pressed in grab mode  */
 int gui_key_grab_command = 0;       /* grab command bound to key?           */
 int gui_key_grab_delay = 0;         /* delay for grab (default is 500)      */
@@ -95,6 +94,7 @@ int gui_key_grab_delay = 0;         /* delay for grab (default is 500)      */
 int *gui_key_buffer = NULL;         /* input buffer (for paste detection)   */
 int gui_key_buffer_alloc = 0;       /* input buffer allocated size          */
 int gui_key_buffer_size = 0;        /* input buffer size in bytes           */
+int gui_key_last_key_pressed_sent = -1;
 
 int gui_key_paste_pending = 0;      /* 1 is big paste was detected and      */
                                     /* WeeChat is asking user what to do    */
@@ -173,9 +173,13 @@ gui_key_get_current_context ()
     if (gui_cursor_mode)
         return GUI_KEY_CONTEXT_CURSOR;
 
-    if (gui_current_window
-        && (gui_current_window->buffer->text_search != GUI_TEXT_SEARCH_DISABLED))
-        return GUI_KEY_CONTEXT_SEARCH;
+    if (gui_current_window)
+    {
+        if (gui_current_window->buffer->text_search == GUI_BUFFER_SEARCH_LINES)
+            return GUI_KEY_CONTEXT_SEARCH;
+        if (gui_current_window->buffer->text_search == GUI_BUFFER_SEARCH_HISTORY)
+            return GUI_KEY_CONTEXT_HISTSEARCH;
+    }
 
     return GUI_KEY_CONTEXT_DEFAULT;
 }
@@ -185,13 +189,12 @@ gui_key_get_current_context ()
  */
 
 void
-gui_key_grab_init (int grab_raw_key, int grab_command, const char *delay)
+gui_key_grab_init (int grab_command, const char *delay)
 {
     long milliseconds;
     char *error;
 
     gui_key_grab = 1;
-    gui_key_grab_raw = grab_raw_key;
     gui_key_grab_count = 0;
     gui_key_grab_command = grab_command;
 
@@ -217,8 +220,7 @@ int
 gui_key_grab_end_timer_cb (const void *pointer, void *data, int remaining_calls)
 {
     char *key_name, *key_name_alias, *key_utf8;
-    const char *ptr_key_name;
-    struct t_gui_key *ptr_key;
+    struct t_gui_key *ptr_key_raw, *ptr_key;
     int rc;
 
     /* make C compiler happy */
@@ -264,23 +266,23 @@ gui_key_grab_end_timer_cb (const void *pointer, void *data, int remaining_calls)
             }
         }
 
-        ptr_key_name = (gui_key_grab_raw) ? key_name : key_name_alias;
-
         /* add expanded key to input buffer */
         if (gui_current_window->buffer->input)
         {
-            gui_input_insert_string (gui_current_window->buffer, ptr_key_name);
-            if (gui_key_grab_command)
+            ptr_key_raw = gui_key_search (gui_keys[GUI_KEY_CONTEXT_DEFAULT],
+                                          key_name);
+            ptr_key = gui_key_search (gui_keys[GUI_KEY_CONTEXT_DEFAULT],
+                                      key_name_alias);
+            gui_input_insert_string (gui_current_window->buffer,
+                                     (ptr_key_raw) ? key_name : key_name_alias);
+            /* add command bound to key (if found) */
+            if (gui_key_grab_command && (ptr_key_raw || ptr_key))
             {
-                /* add command bound to key (if found) */
-                ptr_key = gui_key_search (gui_keys[GUI_KEY_CONTEXT_DEFAULT],
-                                          ptr_key_name);
-                if (ptr_key)
-                {
-                    gui_input_insert_string (gui_current_window->buffer, " ");
-                    gui_input_insert_string (gui_current_window->buffer,
-                                             ptr_key->command);
-                }
+                gui_input_insert_string (gui_current_window->buffer, " ");
+                gui_input_insert_string (
+                    gui_current_window->buffer,
+                    (ptr_key_raw) ?
+                    ptr_key_raw->command : ptr_key->command);
             }
             gui_input_text_changed_modifier_and_signal (
                 gui_current_window->buffer,
@@ -1655,6 +1657,10 @@ gui_key_search_part (struct t_gui_buffer *buffer, int context,
     for (ptr_key = (buffer) ? buffer->keys : gui_keys[context]; ptr_key;
          ptr_key = ptr_key->next_key)
     {
+        /* ignore keys with no command */
+        if (!ptr_key->command || !ptr_key->command[0])
+            continue;
+
         if (ptr_key->key
             && ((context != GUI_KEY_CONTEXT_CURSOR)
                 || (ptr_key->key[0] != '@')))
@@ -2004,6 +2010,31 @@ gui_key_focus_matching (struct t_gui_key *key,
     return match[0] && match[1];
 }
 
+/*
+ * Displays focus hashtable (for debug).
+ */
+
+void
+gui_key_focus_display_hashtable (struct t_hashtable *hashtable)
+{
+    struct t_weelist *list_keys;
+    struct t_weelist_item *ptr_item;
+
+    gui_chat_printf (NULL, _("Hashtable focus:"));
+    list_keys = hashtable_get_list_keys (hashtable);
+    if (list_keys)
+    {
+        for (ptr_item = list_keys->items; ptr_item;
+             ptr_item = ptr_item->next_item)
+        {
+            gui_chat_printf (NULL, "  %s: \"%s\"",
+                             ptr_item->data,
+                             hashtable_get (hashtable, ptr_item->data));
+        }
+        weelist_free (list_keys);
+    }
+}
+
 /*
  * Runs command according to focus.
  *
@@ -2022,8 +2053,6 @@ gui_key_focus_command (const char *key, int context,
     char *command, **commands;
     const char *str_buffer;
     struct t_hashtable *hashtable;
-    struct t_weelist *list_keys;
-    struct t_weelist_item *ptr_item;
     struct t_gui_buffer *ptr_buffer;
 
     debug = 0;
@@ -2090,24 +2119,10 @@ gui_key_focus_command (const char *key, int context,
             gui_input_delete_line (gui_current_window->buffer);
         }
 
-        if (debug > 1)
-        {
-            gui_chat_printf (NULL, _("Hashtable focus:"));
-            list_keys = hashtable_get_list_keys (hashtable);
-            if (list_keys)
-            {
-                for (ptr_item = list_keys->items; ptr_item;
-                     ptr_item = ptr_item->next_item)
-                {
-                    gui_chat_printf (NULL, "  %s: \"%s\"",
-                                     ptr_item->data,
-                                     hashtable_get (hashtable, ptr_item->data));
-                }
-                weelist_free (list_keys);
-            }
-        }
         if (debug)
         {
+            if (debug > 1)
+                gui_key_focus_display_hashtable (hashtable);
             gui_chat_printf (NULL, _("Command for key: \"%s\""),
                              ptr_key->command);
         }
@@ -2158,6 +2173,17 @@ gui_key_focus_command (const char *key, int context,
         return 1;
     }
 
+    if (debug > 1)
+    {
+        hashtable = hook_focus_get_data (hashtable_focus[0],
+                                         hashtable_focus[1]);
+        if (hashtable)
+        {
+            gui_key_focus_display_hashtable (hashtable);
+            hashtable_free (hashtable);
+        }
+    }
+
     return 0;
 }
 
@@ -2391,6 +2417,9 @@ gui_key_pressed (const char *key_str)
 
     rc_expand = gui_key_expand (gui_key_combo, &key_name, &key_name_alias);
 
+    if (!rc_expand)
+        goto end_no_input;
+
     ptr_key = NULL;
     exact_match = 0;
 
@@ -2412,7 +2441,7 @@ gui_key_pressed (const char *key_str)
             /* look for key combo in key table for current buffer */
             ptr_key = gui_key_search_part (
                 gui_current_window->buffer,
-                GUI_KEY_CONTEXT_DEFAULT,
+                context,
                 (const char **)chunks1, chunks1_count,
                 (const char **)chunks2, chunks2_count,
                 &exact_match);
@@ -2421,21 +2450,23 @@ gui_key_pressed (const char *key_str)
             {
                 ptr_key = gui_key_search_part (
                     NULL,
-                    GUI_KEY_CONTEXT_DEFAULT,
+                    context,
                     (const char **)chunks1, chunks1_count,
                     (const char **)chunks2, chunks2_count,
                     &exact_match);
             }
             break;
         case GUI_KEY_CONTEXT_SEARCH:
+        case GUI_KEY_CONTEXT_HISTSEARCH:
             ptr_key = gui_key_search_part (
                 NULL,
-                GUI_KEY_CONTEXT_SEARCH,
+                context,
                 (const char **)chunks1, chunks1_count,
                 (const char **)chunks2, chunks2_count,
                 &exact_match);
             if (!ptr_key)
             {
+                /* fallback to default context */
                 ptr_key = gui_key_search_part (
                     NULL,
                     GUI_KEY_CONTEXT_DEFAULT,
@@ -2447,7 +2478,7 @@ gui_key_pressed (const char *key_str)
         case GUI_KEY_CONTEXT_CURSOR:
             ptr_key = gui_key_search_part (
                 NULL,
-                GUI_KEY_CONTEXT_CURSOR,
+                context,
                 (const char **)chunks1, chunks1_count,
                 (const char **)chunks2, chunks2_count,
                 &exact_match);
@@ -2693,6 +2724,7 @@ gui_key_buffer_reset ()
         gui_key_buffer_optimize ();
     }
     gui_key_paste_lines = 0;
+    gui_key_last_key_pressed_sent = -1;
 }
 
 /*
