@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2006-2007 Emmanuel Bouthenot <kolter@openics.org>
- * Copyright (C) 2006-2015 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2006-2024 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -15,19 +15,21 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
-#ifndef WEECHAT_LUA_API_H
-#define WEECHAT_LUA_API_H 1
+#ifndef WEECHAT_PLUGIN_LUA_API_H
+#define WEECHAT_PLUGIN_LUA_API_H
 
 extern struct luaL_Reg weechat_lua_api_funcs[];
 extern struct t_lua_const weechat_lua_api_consts[];
 
-extern int weechat_lua_api_buffer_input_data_cb (void *data,
+extern int weechat_lua_api_buffer_input_data_cb (const void *pointer,
+                                                 void *data,
                                                  struct t_gui_buffer *buffer,
                                                  const char *input_data);
-extern int weechat_lua_api_buffer_close_cb (void *data,
+extern int weechat_lua_api_buffer_close_cb (const void *pointer,
+                                            void *data,
                                             struct t_gui_buffer *buffer);
 
-#endif /* WEECHAT_LUA_API_H */
+#endif /* WEECHAT_PLUGIN_LUA_API_H */
