@@ -1,7 +1,7 @@
 /*
  * plugin-api-info.c - extra info functions for plugin API
  *
- * Copyright (C) 2003-2023 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2024 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -54,6 +54,7 @@
 #include "../gui/gui-key.h"
 #include "../gui/gui-layout.h"
 #include "../gui/gui-line.h"
+#include "../gui/gui-mouse.h"
 #include "../gui/gui-nick.h"
 #include "../gui/gui-nicklist.h"
 #include "../gui/gui-window.h"
@@ -135,7 +136,7 @@ plugin_api_info_date_cb (const void *pointer, void *data,
                          const char *info_name,
                          const char *arguments)
 {
-    const char *version;
+    const char *compilation_date;
 
     /* make C compiler happy */
     (void) pointer;
@@ -143,8 +144,8 @@ plugin_api_info_date_cb (const void *pointer, void *data,
     (void) info_name;
     (void) arguments;
 
-    version = version_get_compilation_date_time ();
-    return (version) ? strdup (version) : NULL;
+    compilation_date = version_get_compilation_date_time ();
+    return (compilation_date) ? strdup (compilation_date) : NULL;
 }
 
 /*
@@ -197,7 +198,7 @@ plugin_api_info_absolute_path (const char *directory)
 {
     char absolute_path[PATH_MAX];
 
-    if (!realpath (directory, absolute_path))
+    if (!directory || !realpath (directory, absolute_path))
         return NULL;
     return strdup ((absolute_path[0]) ? absolute_path : directory);
 }
@@ -624,6 +625,27 @@ plugin_api_info_cursor_mode_cb (const void *pointer, void *data,
     return strdup (value);
 }
 
+/*
+ * Returns WeeChat info "mouse".
+ */
+
+char *
+plugin_api_info_mouse_cb (const void *pointer, void *data,
+                          const char *info_name,
+                          const char *arguments)
+{
+    char value[32];
+
+    /* make C compiler happy */
+    (void) pointer;
+    (void) data;
+    (void) info_name;
+    (void) arguments;
+
+    snprintf (value, sizeof (value), "%d", gui_mouse_enabled);
+    return strdup (value);
+}
+
 /*
  * Returns WeeChat info "term_width".
  */
@@ -1645,24 +1667,24 @@ plugin_api_infolist_key_cb (const void *pointer, void *data,
     (void) infolist_name;
     (void) obj_pointer;
 
-    ptr_infolist = infolist_new (NULL);
-    if (!ptr_infolist)
-        return NULL;
-
     if (arguments && arguments[0])
         context = gui_key_search_context (arguments);
     else
         context = GUI_KEY_CONTEXT_DEFAULT;
-    if (context >= 0)
+    if (context < 0)
+        return NULL;
+
+    ptr_infolist = infolist_new (NULL);
+    if (!ptr_infolist)
+        return NULL;
+
+    for (ptr_key = gui_keys[context]; ptr_key;
+         ptr_key = ptr_key->next_key)
     {
-        for (ptr_key = gui_keys[context]; ptr_key;
-             ptr_key = ptr_key->next_key)
+        if (!gui_key_add_to_infolist (ptr_infolist, ptr_key))
         {
-            if (!gui_key_add_to_infolist (ptr_infolist, ptr_key))
-            {
-                infolist_free (ptr_infolist);
-                return NULL;
-            }
+            infolist_free (ptr_infolist);
+            return NULL;
         }
     }
     return ptr_infolist;
@@ -2125,6 +2147,9 @@ plugin_api_info_init ()
     hook_info (NULL, "cursor_mode",
                N_("1 if cursor mode is enabled"),
                NULL, &plugin_api_info_cursor_mode_cb, NULL, NULL);
+    hook_info (NULL, "mouse",
+               N_("1 if mouse is enabled"),
+               NULL, &plugin_api_info_mouse_cb, NULL, NULL);
     hook_info (NULL, "term_width",
                N_("width of terminal"),
                NULL, &plugin_api_info_term_width_cb, NULL, NULL);
