@@ -1,7 +1,7 @@
 /*
  * relay-weechat-protocol.c - WeeChat protocol for relay to client
  *
- * Copyright (C) 2003-2021 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2024 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -36,21 +36,6 @@
 #include "../relay-raw.h"
 
 
-/*
- * Checks if the buffer pointer is a relay buffer (relay raw/list).
- *
- * Returns:
- *   1: buffer is a relay buffer (raw/list)
- *   0: buffer is NOT a relay buffer
- */
-
-int
-relay_weechat_is_relay_buffer (struct t_gui_buffer *buffer)
-{
-    return ((relay_raw_buffer && (buffer == relay_raw_buffer))
-            || (relay_buffer && (buffer == relay_buffer))) ? 1 : 0;
-}
-
 /*
  * Gets buffer pointer with argument from a command.
  *
@@ -66,7 +51,6 @@ relay_weechat_protocol_get_buffer (const char *arg)
     struct t_gui_buffer *ptr_buffer;
     unsigned long value;
     int rc;
-    struct t_hdata *ptr_hdata;
 
     ptr_buffer = NULL;
 
@@ -77,10 +61,10 @@ relay_weechat_protocol_get_buffer (const char *arg)
             ptr_buffer = (struct t_gui_buffer *)value;
         if (ptr_buffer)
         {
-            ptr_hdata = weechat_hdata_get ("buffer");
-            if (!weechat_hdata_check_pointer (ptr_hdata,
-                                              weechat_hdata_get_list (ptr_hdata, "gui_buffers"),
-                                              ptr_buffer))
+            if (!weechat_hdata_check_pointer (
+                    relay_hdata_buffer,
+                    weechat_hdata_get_list (relay_hdata_buffer, "gui_buffers"),
+                    ptr_buffer))
             {
                 /* invalid pointer! */
                 ptr_buffer = NULL;
@@ -203,6 +187,10 @@ relay_weechat_protocol_handshake_reply (struct t_relay_client *client,
             hashtable,
             "compression",
             relay_weechat_compression_string[RELAY_WEECHAT_DATA(client, compression)]);
+        weechat_hashtable_set (
+            hashtable,
+            "escape_commands",
+            RELAY_WEECHAT_DATA(client, escape_commands) ? "on" : "off");
 
         msg = relay_weechat_msg_new (id);
         if (msg)
@@ -229,7 +217,7 @@ relay_weechat_protocol_handshake_reply (struct t_relay_client *client,
 
 RELAY_WEECHAT_PROTOCOL_CALLBACK(handshake)
 {
-    char **options, **auths, *pos;
+    char **options, **auths, **compressions, *pos;
     int i, j, index_hash_algo, hash_algo_found, auth_allowed, compression;
     int password_received, plain_text_password;
 
@@ -292,9 +280,34 @@ RELAY_WEECHAT_PROTOCOL_CALLBACK(handshake)
                 }
                 else if (strcmp (options[i], "compression") == 0)
                 {
-                    compression = relay_weechat_compression_search (pos);
-                    if (compression >= 0)
-                        RELAY_WEECHAT_DATA(client, compression) = compression;
+                    compressions = weechat_string_split (
+                        pos,
+                        ":",
+                        NULL,
+                        WEECHAT_STRING_SPLIT_STRIP_LEFT
+                        | WEECHAT_STRING_SPLIT_STRIP_RIGHT
+                        | WEECHAT_STRING_SPLIT_COLLAPSE_SEPS,
+                        0,
+                        NULL);
+                    if (compressions)
+                    {
+                        for (j = 0; compressions[j]; j++)
+                        {
+                            compression = relay_weechat_compression_search (compressions[j]);
+                            if (compression >= 0)
+                            {
+                                RELAY_WEECHAT_DATA(client, compression) = compression;
+                                break;
+                            }
+                        }
+                        weechat_string_free_split (compressions);
+                    }
+                }
+                else if (strcmp (options[i], "escape_commands") == 0)
+                {
+                    RELAY_WEECHAT_DATA(client, escape_commands) =
+                        (weechat_strcmp (pos, "on") == 0) ?
+                        1 : 0;
                 }
             }
         }
@@ -330,19 +343,16 @@ RELAY_WEECHAT_PROTOCOL_CALLBACK(handshake)
  * Format is:  init arg1=value1,arg2=value2
  *
  * Allowed arguments:
- *   password       plain text password (recommended with SSL only)
+ *   password       plain text password (recommended with TLS only)
  *   password_hash  hashed password, value is: algorithm:[parameters:]hash
  *                  supported algorithms: sha256, sha512 and pbkdf2
  *                  for pbkdf2, parameters are: algorithm, salt, iterations
  *                  hash is given in hexadecimal
  *   totp           time-based one time password used as secondary
  *                  authentication factor
- *   compression    zlib (default) or off
  *
  * Message looks like:
  *   init password=mypass
- *   init password=mypass,compression=zlib
- *   init password=mypass,compression=off
  *   init password_hash=sha256:71c480df93d6ae2f1efad1447c66c9…,totp=123456
  *   init password_hash=pbkdf2:sha256:414232…:100000:01757d53157c…,totp=123456
  */
@@ -350,7 +360,7 @@ RELAY_WEECHAT_PROTOCOL_CALLBACK(handshake)
 RELAY_WEECHAT_PROTOCOL_CALLBACK(init)
 {
     char **options, *pos, *relay_password, *totp_secret, *info_totp_args, *info_totp;
-    int i, compression, length, password_received, totp_received;
+    int i, length, password_received, totp_received;
 
     RELAY_WEECHAT_PROTOCOL_MIN_ARGS(0);
 
@@ -411,12 +421,6 @@ RELAY_WEECHAT_PROTOCOL_CALLBACK(init)
                         }
                     }
                 }
-                else if (strcmp (options[i], "compression") == 0)
-                {
-                    compression = relay_weechat_compression_search (pos);
-                    if (compression >= 0)
-                        RELAY_WEECHAT_DATA(client, compression) = compression;
-                }
             }
         }
         weechat_string_free_split_command (options);
@@ -653,6 +657,7 @@ RELAY_WEECHAT_PROTOCOL_CALLBACK(input)
             "commands",
             weechat_config_string (relay_config_weechat_commands));
     }
+
     /*
      * delay the execution of command after we go back in the WeeChat
      * main loop (some commands like /upgrade executed now can cause
@@ -682,7 +687,6 @@ RELAY_WEECHAT_PROTOCOL_CALLBACK(completion)
     struct t_gui_buffer *ptr_buffer;
     struct t_gui_completion *completion;
     struct t_gui_completion_word *word;
-    struct t_hdata *ptr_hdata_completion, *ptr_hdata_completion_word;
     struct t_arraylist *ptr_list;
     struct t_relay_weechat_msg *msg;
     char *error, *pos_data;
@@ -735,15 +739,7 @@ RELAY_WEECHAT_PROTOCOL_CALLBACK(completion)
         goto error;
     }
 
-    ptr_hdata_completion = weechat_hdata_get ("completion");
-    if (!ptr_hdata_completion)
-        goto error;
-
-    ptr_hdata_completion_word = weechat_hdata_get ("completion_word");
-    if (!ptr_hdata_completion_word)
-        goto error;
-
-    ptr_list = weechat_hdata_pointer (ptr_hdata_completion, completion, "list");
+    ptr_list = weechat_hdata_pointer (relay_hdata_completion, completion, "list");
     if (!ptr_list)
         goto error;
 
@@ -762,7 +758,7 @@ RELAY_WEECHAT_PROTOCOL_CALLBACK(completion)
         relay_weechat_msg_add_int (msg, 1); /* count */
         relay_weechat_msg_add_pointer (msg, completion);
         /* context */
-        context = weechat_hdata_integer (ptr_hdata_completion, completion,
+        context = weechat_hdata_integer (relay_hdata_completion, completion,
                                          "context");
         switch (context)
         {
@@ -782,10 +778,10 @@ RELAY_WEECHAT_PROTOCOL_CALLBACK(completion)
         /* base_word */
         relay_weechat_msg_add_string (
             msg,
-            weechat_hdata_string (ptr_hdata_completion,
+            weechat_hdata_string (relay_hdata_completion,
                                   completion, "base_word"));
         /* pos_start */
-        pos_start = weechat_hdata_integer (ptr_hdata_completion,
+        pos_start = weechat_hdata_integer (relay_hdata_completion,
                                            completion, "position_replace");
         relay_weechat_msg_add_int (msg, pos_start);
 
@@ -796,7 +792,7 @@ RELAY_WEECHAT_PROTOCOL_CALLBACK(completion)
         /* add_space */
         relay_weechat_msg_add_int (
             msg,
-            weechat_hdata_integer (ptr_hdata_completion,
+            weechat_hdata_integer (relay_hdata_completion,
                                    completion, "add_space"));
         /* list */
         relay_weechat_msg_add_type (msg, RELAY_WEECHAT_MSG_OBJ_STRING);
@@ -808,7 +804,7 @@ RELAY_WEECHAT_PROTOCOL_CALLBACK(completion)
                 ptr_list, i);
             relay_weechat_msg_add_string (
                 msg,
-                weechat_hdata_string (ptr_hdata_completion_word, word, "word"));
+                weechat_hdata_string (relay_hdata_completion_word, word, "word"));
         }
 
         /* send message */
@@ -852,7 +848,6 @@ relay_weechat_protocol_signal_buffer_cb (const void *pointer, void *data,
 {
     struct t_relay_client *ptr_client;
     struct t_gui_line *ptr_line;
-    struct t_hdata *ptr_hdata_line, *ptr_hdata_line_data;
     struct t_gui_line_data *ptr_line_data;
     struct t_gui_buffer *ptr_buffer;
     struct t_relay_weechat_msg *msg;
@@ -1086,7 +1081,7 @@ relay_weechat_protocol_signal_buffer_cb (const void *pointer, void *data,
     else if (strcmp (signal, "buffer_cleared") == 0)
     {
         ptr_buffer = (struct t_gui_buffer *)signal_data;
-        if (!ptr_buffer || relay_weechat_is_relay_buffer (ptr_buffer))
+        if (!ptr_buffer || relay_buffer_is_relay (ptr_buffer))
             return WEECHAT_RC_OK;
 
         /* send signal only if sync with flag "buffer" */
@@ -1111,21 +1106,13 @@ relay_weechat_protocol_signal_buffer_cb (const void *pointer, void *data,
         if (!ptr_line)
             return WEECHAT_RC_OK;
 
-        ptr_hdata_line = weechat_hdata_get ("line");
-        if (!ptr_hdata_line)
-            return WEECHAT_RC_OK;
-
-        ptr_hdata_line_data = weechat_hdata_get ("line_data");
-        if (!ptr_hdata_line_data)
-            return WEECHAT_RC_OK;
-
-        ptr_line_data = weechat_hdata_pointer (ptr_hdata_line, ptr_line, "data");
+        ptr_line_data = weechat_hdata_pointer (relay_hdata_line, ptr_line, "data");
         if (!ptr_line_data)
             return WEECHAT_RC_OK;
 
-        ptr_buffer = weechat_hdata_pointer (ptr_hdata_line_data, ptr_line_data,
+        ptr_buffer = weechat_hdata_pointer (relay_hdata_line_data, ptr_line_data,
                                             "buffer");
-        if (!ptr_buffer || relay_weechat_is_relay_buffer (ptr_buffer))
+        if (!ptr_buffer || relay_buffer_is_relay (ptr_buffer))
             return WEECHAT_RC_OK;
 
         /* send signal only if sync with flag "buffer" */
@@ -1138,11 +1125,11 @@ relay_weechat_protocol_signal_buffer_cb (const void *pointer, void *data,
                 snprintf (cmd_hdata, sizeof (cmd_hdata),
                           "line_data:0x%lx",
                           (unsigned long)ptr_line_data);
-                relay_weechat_msg_add_hdata (msg, cmd_hdata,
-                                             "buffer,date,date_printed,"
-                                             "displayed,notify_level,"
-                                             "highlight,tags_array,prefix,"
-                                             "message");
+                relay_weechat_msg_add_hdata (
+                    msg, cmd_hdata,
+                    "buffer,date,date_usec,date_printed,date_usec_printed,"
+                    "displayed,notify_level,highlight,tags_array,"
+                    "prefix,message");
                 relay_weechat_msg_send (ptr_client, msg);
                 relay_weechat_msg_free (msg);
             }
@@ -1197,7 +1184,6 @@ relay_weechat_protocol_nicklist_map_cb (void *data,
     struct t_relay_client *ptr_client;
     struct t_gui_buffer *ptr_buffer;
     struct t_relay_weechat_nicklist *ptr_nicklist;
-    struct t_hdata *ptr_hdata;
     struct t_relay_weechat_msg *msg;
 
     /* make C compiler happy */
@@ -1207,32 +1193,29 @@ relay_weechat_protocol_nicklist_map_cb (void *data,
     ptr_buffer = (struct t_gui_buffer *)key;
     ptr_nicklist = (struct t_relay_weechat_nicklist *)value;
 
-    ptr_hdata = weechat_hdata_get ("buffer");
-    if (ptr_hdata)
+    if (weechat_hdata_check_pointer (
+            relay_hdata_buffer,
+            weechat_hdata_get_list (relay_hdata_buffer, "gui_buffers"),
+            ptr_buffer))
     {
-        if (weechat_hdata_check_pointer (ptr_hdata,
-                                         weechat_hdata_get_list (ptr_hdata, "gui_buffers"),
-                                         ptr_buffer))
+        /*
+         * if no diff at all, or if diffs are bigger than nicklist:
+         * send whole nicklist
+         */
+        if (ptr_nicklist
+            && ((ptr_nicklist->items_count == 0)
+                || (ptr_nicklist->items_count >= weechat_buffer_get_integer (ptr_buffer, "nicklist_count") + 1)))
         {
-            /*
-             * if no diff at all, or if diffs are bigger than nicklist:
-             * send whole nicklist
-             */
-            if (ptr_nicklist
-                && ((ptr_nicklist->items_count == 0)
-                    || (ptr_nicklist->items_count >= weechat_buffer_get_integer (ptr_buffer, "nicklist_count") + 1)))
-            {
-                ptr_nicklist = NULL;
-            }
+            ptr_nicklist = NULL;
+        }
 
-            /* send nicklist diffs or full nicklist */
-            msg = relay_weechat_msg_new ((ptr_nicklist) ? "_nicklist_diff" : "_nicklist");
-            if (msg)
-            {
-                relay_weechat_msg_add_nicklist (msg, ptr_buffer, ptr_nicklist);
-                relay_weechat_msg_send (ptr_client, msg);
-                relay_weechat_msg_free (msg);
-            }
+        /* send nicklist diffs or full nicklist */
+        msg = relay_weechat_msg_new ((ptr_nicklist) ? "_nicklist_diff" : "_nicklist");
+        if (msg)
+        {
+            relay_weechat_msg_add_nicklist (msg, ptr_buffer, ptr_nicklist);
+            relay_weechat_msg_send (ptr_client, msg);
+            relay_weechat_msg_free (msg);
         }
     }
 }
@@ -1732,7 +1715,8 @@ RELAY_WEECHAT_PROTOCOL_CALLBACK(quit)
 void
 relay_weechat_protocol_recv (struct t_relay_client *client, const char *data)
 {
-    char *pos, *id, *command, **argv, **argv_eol;
+    const char *ptr_data;
+    char *data_unescaped, *pos, *id, *command, **argv, **argv_eol;
     int i, argc, return_code;
     struct t_relay_weechat_protocol_cb protocol_cb[] =
         { { "handshake", &relay_weechat_protocol_cb_handshake },
@@ -1765,35 +1749,44 @@ relay_weechat_protocol_recv (struct t_relay_client *client, const char *data)
                         data);
     }
 
-    /* extract id */
+    data_unescaped = NULL;
+    ptr_data = data;
     id = NULL;
-    if (data[0] == '(')
+    command = NULL;
+    argv = NULL;
+    argv_eol = NULL;
+
+    if (RELAY_WEECHAT_DATA(client, escape_commands))
     {
-        pos = strchr (data, ')');
+        data_unescaped = weechat_string_convert_escaped_chars (data);
+        if (data_unescaped)
+            ptr_data = data_unescaped;
+    }
+
+    /* extract id */
+    if (ptr_data[0] == '(')
+    {
+        pos = strchr (ptr_data, ')');
         if (pos)
         {
-            id = weechat_strndup (data + 1, pos - data - 1);
-            data = pos + 1;
-            while (data[0] == ' ')
+            id = weechat_strndup (ptr_data + 1, pos - ptr_data - 1);
+            ptr_data = pos + 1;
+            while (ptr_data[0] == ' ')
             {
-                data++;
+                ptr_data++;
             }
         }
     }
 
     /* search end of data */
-    pos = strchr (data, ' ');
+    pos = strchr (ptr_data, ' ');
     if (pos)
-        command = weechat_strndup (data, pos - data);
+        command = weechat_strndup (ptr_data, pos - ptr_data);
     else
-        command = strdup (data);
+        command = strdup (ptr_data);
 
     if (!command)
-    {
-        if (id)
-            free (id);
-        return;
-    }
+        goto end;
 
     argc = 0;
     argv = NULL;
@@ -1858,6 +1851,9 @@ relay_weechat_protocol_recv (struct t_relay_client *client, const char *data)
         }
     }
 
+end:
+    if (data_unescaped)
+        free (data_unescaped);
     if (id)
         free (id);
     free (command);
