@@ -1,7 +1,7 @@
 /*
  * test-core-eval.cpp - test evaluation functions
  *
- * Copyright (C) 2014-2021 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2014-2024 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -453,7 +453,7 @@ TEST(CoreEval, EvalExpression)
 {
     struct t_hashtable *pointers, *extra_vars, *options;
     struct t_config_option *ptr_option;
-    char *value, str_value[256];
+    char *value, str_value[256], str_expr[256];
     const char *ptr_debug_output;
 
     pointers = hashtable_new (32,
@@ -489,12 +489,89 @@ TEST(CoreEval, EvalExpression)
     WEE_CHECK_EVAL("", "${}");
     WEE_CHECK_EVAL("", "${xyz}");
 
+    /* test raw string with syntax highlighting */
+    WEE_CHECK_EVAL("", "${raw_hl:}");
+    WEE_CHECK_EVAL("test", "${raw_hl:test}");
+    WEE_CHECK_EVAL("\\${", "${raw_hl:\\${");
+    snprintf (str_value, sizeof (str_value),
+              "%s${info:version}%s",
+              gui_color_get_custom (config_eval_syntax_colors[0]),
+              gui_color_get_custom ("reset"));
+    WEE_CHECK_EVAL(str_value, "${raw_hl:${info:version}}");
+    snprintf (str_value, sizeof (str_value),
+              "test_%s${info:version}%s_end",
+              gui_color_get_custom (config_eval_syntax_colors[0]),
+              gui_color_get_custom ("reset"));
+    WEE_CHECK_EVAL(str_value, "test_${raw_hl:${info:version}}_end");
+    snprintf (str_value, sizeof (str_value),
+              "test_%s${cut:3,,%s${rev:%s${info:version}%s}%s}%s_end",
+              gui_color_get_custom (config_eval_syntax_colors[0]),
+              gui_color_get_custom (config_eval_syntax_colors[1]),
+              gui_color_get_custom (config_eval_syntax_colors[2]),
+              gui_color_get_custom (config_eval_syntax_colors[1]),
+              gui_color_get_custom (config_eval_syntax_colors[0]),
+              gui_color_get_custom ("reset"));
+    WEE_CHECK_EVAL(str_value, "test_${raw_hl:${cut:3,,${rev:${info:version}}}}_end");
+    snprintf (str_value, sizeof (str_value),
+              "test_%s${raw:${test}}%s_end",
+              gui_color_get_custom (config_eval_syntax_colors[0]),
+              gui_color_get_custom ("reset"));
+    WEE_CHECK_EVAL(str_value, "test_${raw_hl:${raw:${test}}}_end");
+    snprintf (str_value, sizeof (str_value),
+              "test_%s${if:a==1?%s${yes}%s:%s${no}%s}%s_end",
+              gui_color_get_custom (config_eval_syntax_colors[0]),
+              gui_color_get_custom (config_eval_syntax_colors[1]),
+              gui_color_get_custom (config_eval_syntax_colors[0]),
+              gui_color_get_custom (config_eval_syntax_colors[1]),
+              gui_color_get_custom (config_eval_syntax_colors[0]),
+              gui_color_get_custom ("reset"));
+    WEE_CHECK_EVAL(str_value, "test_${raw_hl:${if:a==1?${yes}:${no}}}_end");
+    snprintf (str_value, sizeof (str_value),
+              "test_%s${a:%s${b:%s${c:%s${d:%s${e:%s${f:%s${g:%s${h:%s${i:}%s}%s}%s}%s}%s}%s}%s}%s}%s_end",
+              gui_color_get_custom (config_eval_syntax_colors[0 % config_num_eval_syntax_colors]),
+              gui_color_get_custom (config_eval_syntax_colors[1 % config_num_eval_syntax_colors]),
+              gui_color_get_custom (config_eval_syntax_colors[2 % config_num_eval_syntax_colors]),
+              gui_color_get_custom (config_eval_syntax_colors[3 % config_num_eval_syntax_colors]),
+              gui_color_get_custom (config_eval_syntax_colors[4 % config_num_eval_syntax_colors]),
+              gui_color_get_custom (config_eval_syntax_colors[5 % config_num_eval_syntax_colors]),
+              gui_color_get_custom (config_eval_syntax_colors[6 % config_num_eval_syntax_colors]),
+              gui_color_get_custom (config_eval_syntax_colors[7 % config_num_eval_syntax_colors]),
+              gui_color_get_custom (config_eval_syntax_colors[8 % config_num_eval_syntax_colors]),
+              gui_color_get_custom (config_eval_syntax_colors[7 % config_num_eval_syntax_colors]),
+              gui_color_get_custom (config_eval_syntax_colors[6 % config_num_eval_syntax_colors]),
+              gui_color_get_custom (config_eval_syntax_colors[5 % config_num_eval_syntax_colors]),
+              gui_color_get_custom (config_eval_syntax_colors[4 % config_num_eval_syntax_colors]),
+              gui_color_get_custom (config_eval_syntax_colors[3 % config_num_eval_syntax_colors]),
+              gui_color_get_custom (config_eval_syntax_colors[2 % config_num_eval_syntax_colors]),
+              gui_color_get_custom (config_eval_syntax_colors[1 % config_num_eval_syntax_colors]),
+              gui_color_get_custom (config_eval_syntax_colors[0 % config_num_eval_syntax_colors]),
+              gui_color_get_custom ("reset"));
+    WEE_CHECK_EVAL(str_value,
+                   "test_${raw_hl:${a:${b:${c:${d:${e:${f:${g:${h:${i:}}}}}}}}}}_end");
+
     /* test raw string */
+    WEE_CHECK_EVAL("", "${raw:}");
+    WEE_CHECK_EVAL("test", "${raw:test}");
+    WEE_CHECK_EVAL("\\${", "${raw:\\${");
     WEE_CHECK_EVAL("${info:version}", "${raw:${info:version}}");
     WEE_CHECK_EVAL("yes", "${if:${raw:test?}==${raw:test?}?yes:no}");
     WEE_CHECK_EVAL("no", "${if:${raw:test?}==${raw:test}?yes:no}");
     WEE_CHECK_EVAL("16", "${length:${raw:${buffer.number}}}");
 
+    /* test string with syntax highlighting */
+    WEE_CHECK_EVAL("", "${hl:}");
+    WEE_CHECK_EVAL("test", "${hl:test}");
+    snprintf (str_value, sizeof (str_value),
+              "%s${info:version}%s",
+              gui_color_get_custom (config_eval_syntax_colors[0]),
+              gui_color_get_custom ("reset"));
+    WEE_CHECK_EVAL("test_weechat_end", "test_${hl:${buffer.name}}_end");
+    snprintf (str_value, sizeof (str_value),
+              "test_%s${buffer.name}%s_end",
+              gui_color_get_custom (config_eval_syntax_colors[0]),
+              gui_color_get_custom ("reset"));
+    WEE_CHECK_EVAL(str_value, "test_${hl:${raw:${buffer.name}}}_end");
+
     /* test eval of substring */
     WEE_CHECK_EVAL("\t", "${eval:${\\t}}");
 
@@ -516,6 +593,35 @@ TEST(CoreEval, EvalExpression)
     WEE_CHECK_EVAL("\t", "${\\t}");
     WEE_CHECK_EVAL("\t", "${esc:\t}");
 
+    /* test range of chars */
+    WEE_CHECK_EVAL("0123456789", "${chars:digit}");
+    WEE_CHECK_EVAL("0123456789abcdefABCDEF", "${chars:xdigit}");
+    WEE_CHECK_EVAL("abcdefghijklmnopqrstuvwxyz", "${chars:lower}");
+    WEE_CHECK_EVAL("ABCDEFGHIJKLMNOPQRSTUVWXYZ", "${chars:upper}");
+    WEE_CHECK_EVAL("abcdefghijklmnopqrstuvwxyz"
+                   "ABCDEFGHIJKLMNOPQRSTUVWXYZ", "${chars:alpha}");
+    WEE_CHECK_EVAL("abcdefghijklmnopqrstuvwxyz"
+                   "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
+                   "0123456789", "${chars:alnum}");
+    WEE_CHECK_EVAL("0123456789", "${chars:0-9}");
+    WEE_CHECK_EVAL("abcdefgh", "${chars:a-h}");
+    WEE_CHECK_EVAL("JKLMNOPQRSTUV", "${chars:J-V}");
+    WEE_CHECK_EVAL("é", "${chars:é-é}");
+    WEE_CHECK_EVAL("àáâãäåæçèé", "${chars:à-é}");
+    WEE_CHECK_EVAL("←↑→↓", "${chars:←-↓}");  /* U+2190 - U+2193 */
+    WEE_CHECK_EVAL("▁▂▃▄▅▆▇█▉▊▋▌▍▎▏", "${chars:▁-▏}");  /* U+2581 - U+258F */
+    WEE_CHECK_EVAL("", "${chars:Z-A}");  /* invalid (reverse) */
+
+    /* test case conversion: to lower case */
+    WEE_CHECK_EVAL("", "${lower:}");
+    WEE_CHECK_EVAL("this is a test", "${lower:This is a TEST}");
+    WEE_CHECK_EVAL("testé testé", "${lower:TESTÉ Testé}");
+
+    /* test case conversion: to upper case */
+    WEE_CHECK_EVAL("", "${upper:}");
+    WEE_CHECK_EVAL("THIS IS A TEST", "${upper:This is a TEST}");
+    WEE_CHECK_EVAL("TESTÉ TESTÉ", "${upper:TESTÉ Testé}");
+
     /* test hidden chars */
     WEE_CHECK_EVAL("", "${hide:invalid}");
     WEE_CHECK_EVAL("********", "${hide:*,password}");
@@ -648,6 +754,85 @@ TEST(CoreEval, EvalExpression)
     WEE_CHECK_EVAL("14", "${lengthscr:こんにちは世界}");
     WEE_CHECK_EVAL("14", "${lengthscr:${color:green}こんにちは世界}");
 
+    /* test split of string */
+    WEE_CHECK_EVAL("", "${split:}");
+    WEE_CHECK_EVAL("", "${split:1}");
+    WEE_CHECK_EVAL("", "${split:1,}");
+    WEE_CHECK_EVAL("", "${split:1,,}");
+    WEE_CHECK_EVAL("", "${split:count}");
+    WEE_CHECK_EVAL("", "${split:count,}");
+    WEE_CHECK_EVAL("", "${split:count,,}");
+    WEE_CHECK_EVAL("0", "${split:count,,,}");
+
+    WEE_CHECK_EVAL("abc", "${split:random,,,abc}");
+
+    WEE_CHECK_EVAL("", "${split:0,,,abc,def,ghi}");
+    WEE_CHECK_EVAL("abc", "${split:1,,,abc}");
+    WEE_CHECK_EVAL("abc", "${split:1,,,abc,def}");
+    WEE_CHECK_EVAL("abc", "${split:1,,,abc,def,ghi}");
+    WEE_CHECK_EVAL("def", "${split:2,,,abc,def,ghi}");
+    WEE_CHECK_EVAL("ghi", "${split:3,,,abc,def,ghi}");
+    WEE_CHECK_EVAL("", "${split:4,,,abc,def,ghi}");
+
+    WEE_CHECK_EVAL("ghi", "${split:-1,,,abc,def,ghi}");
+    WEE_CHECK_EVAL("def", "${split:-2,,,abc,def,ghi}");
+    WEE_CHECK_EVAL("abc", "${split:-3,,,abc,def,ghi}");
+    WEE_CHECK_EVAL("", "${split:-4,,,abc,def,ghi}");
+    WEE_CHECK_EVAL("3", "${split:count,,,abc,def,ghi}");
+
+    WEE_CHECK_EVAL("abc", "${split:1, ,,abc def ghi}");
+    WEE_CHECK_EVAL("def", "${split:2, ,,abc def ghi}");
+    WEE_CHECK_EVAL("ghi", "${split:3, ,,abc def ghi}");
+    WEE_CHECK_EVAL("", "${split:4, ,,abc def ghi}");
+    WEE_CHECK_EVAL("3", "${split:count, ,,abc def ghi}");
+
+    WEE_CHECK_EVAL("abc", "${split:1,_-,,abc-def_ghi}");
+    WEE_CHECK_EVAL("def", "${split:2,_-,,abc-def_ghi}");
+    WEE_CHECK_EVAL("ghi", "${split:3,_-,,abc-def_ghi}");
+    WEE_CHECK_EVAL("", "${split:4,_-,,abc-def_ghi}");
+    WEE_CHECK_EVAL("3", "${split:count,_-,,abc-def_ghi}");
+
+    WEE_CHECK_EVAL("abc,def,ghi", "${split:1,,keep_eol,abc,def,ghi}");
+    WEE_CHECK_EVAL("def,ghi", "${split:2,,keep_eol,abc,def,ghi}");
+    WEE_CHECK_EVAL("ghi", "${split:3,,keep_eol,abc,def,ghi}");
+
+    WEE_CHECK_EVAL("abc,def,ghi", "${split:1,,keep_eol+strip_left,,,abc,def,ghi}");
+    WEE_CHECK_EVAL("def,ghi", "${split:2,,keep_eol+strip_left,,,abc,def,ghi}");
+    WEE_CHECK_EVAL("ghi", "${split:3,,keep_eol+strip_left,,,abc,def,ghi}");
+
+    WEE_CHECK_EVAL("abc,def,ghi", "${split:1,,keep_eol+strip_left+strip_right,,,abc,def,ghi,,}");
+    WEE_CHECK_EVAL("def,ghi", "${split:2,,keep_eol+strip_left+strip_right,,,abc,def,ghi,,}");
+    WEE_CHECK_EVAL("ghi", "${split:3,,keep_eol+strip_left+strip_right,,,abc,def,ghi,,}");
+
+    WEE_CHECK_EVAL("abc", "${split:1,,collapse_seps,abc,,,def,,ghi}");
+    WEE_CHECK_EVAL("def", "${split:2,,collapse_seps,abc,,,def,,ghi}");
+    WEE_CHECK_EVAL("ghi", "${split:3,,collapse_seps,abc,,,def,,ghi}");
+
+    WEE_CHECK_EVAL("abc", "${split:1,,strip_items=_,_abc_,_def_,_ghi_}");
+    WEE_CHECK_EVAL("def", "${split:2,,strip_items=_,_abc_,_def_,_ghi_}");
+    WEE_CHECK_EVAL("ghi", "${split:3,,strip_items=_,_abc_,_def_,_ghi_}");
+
+    WEE_CHECK_EVAL("def", "${split:2,,max_items=0,abc,def,ghi}");
+    WEE_CHECK_EVAL("def", "${split:2,,max_items=2,abc,def,ghi}");
+    WEE_CHECK_EVAL("", "${split:2,,max_items=1,abc,def,ghi}");
+
+    /* test split of shell arguments */
+    WEE_CHECK_EVAL("", "${split_shell:}");
+    WEE_CHECK_EVAL("", "${split_shell:1}");
+    WEE_CHECK_EVAL("", "${split_shell:1,}");
+    WEE_CHECK_EVAL("", "${split_shell:count}");
+    WEE_CHECK_EVAL("0", "${split_shell:count,}");
+
+    WEE_CHECK_EVAL("first arg", "${split_shell:random,\"first arg\"}");
+
+    WEE_CHECK_EVAL("", "${split_shell:0,\"first arg\" arg2}");
+    WEE_CHECK_EVAL("first arg", "${split_shell:1,\"first arg\" arg2}");
+    WEE_CHECK_EVAL("arg2", "${split_shell:2,\"first arg\" arg2}");
+
+    WEE_CHECK_EVAL("arg2", "${split_shell:-1,\"first arg\" arg2}");
+    WEE_CHECK_EVAL("first arg", "${split_shell:-2,\"first arg\" arg2}");
+    WEE_CHECK_EVAL("", "${split_shell:-3,\"first arg\" arg2}");
+
     /* test color */
     WEE_CHECK_EVAL(gui_color_get_custom ("green"), "${color:green}");
     WEE_CHECK_EVAL(gui_color_get_custom ("*214"), "${color:*214}");
@@ -677,7 +862,7 @@ TEST(CoreEval, EvalExpression)
                    "${modifier:color_decode_ansi,0,test_\x1B[92mno_color}");
     snprintf (str_value, sizeof (str_value),
               "test_%slightgreen",
-              gui_color_get_custom ("lightgreen"));
+              gui_color_get_custom ("|lightgreen"));
     WEE_CHECK_EVAL(str_value,
                    "${modifier:color_decode_ansi,1,test_\x1B[92mlightgreen}");
     snprintf (str_value, sizeof (str_value),
@@ -720,6 +905,8 @@ TEST(CoreEval, EvalExpression)
     WEE_CHECK_EVAL("yes", "${if:5>2?yes:no}");
     WEE_CHECK_EVAL("no", "${if:1>7?yes:no}");
     WEE_CHECK_EVAL("yes", "${if:5>2 && 6>3?yes:no}");
+    WEE_CHECK_EVAL("yes", "${if:1?yes:no:test}");
+    WEE_CHECK_EVAL("no:test", "${if:0?yes:no:test}");
     WEE_CHECK_EVAL("yes-yes", "${if:5>2?${if:6>3?yes-yes:yes-no}:${if:9>4?no-yes:no-no}}");
     WEE_CHECK_EVAL("yes-no", "${if:5>2?${if:1>7?yes-yes:yes-no}:${if:9>4?no-yes:no-no}}");
     WEE_CHECK_EVAL("no-yes", "${if:1>7?${if:6>3?yes-yes:yes-no}:${if:9>4?no-yes:no-no}}");
@@ -751,10 +938,37 @@ TEST(CoreEval, EvalExpression)
     WEE_CHECK_EVAL("18", "${calc:(5+1)*3}");
     WEE_CHECK_EVAL("123129", "${calc:${repeat:2,123}+2*3}");
 
+    /* test random */
+    WEE_CHECK_EVAL("0", "${random:}");
+    WEE_CHECK_EVAL("0", "${random:1}");
+    WEE_CHECK_EVAL("0", "${random:1,}");
+    WEE_CHECK_EVAL("0", "${random:5,1}");
+    WEE_CHECK_EVAL("2", "${random:2,2}");
+    snprintf (str_expr, sizeof (str_expr),
+              "${random:-1,%ld}",
+              (long)RAND_MAX);
+    WEE_CHECK_EVAL("0", str_expr);
+    snprintf (str_expr, sizeof (str_expr),
+              "${random:%ld,%ld}",
+              (long)RAND_MAX,
+              (long)RAND_MAX);
+    snprintf (str_value, sizeof (str_value),
+              "%ld",
+              (long)RAND_MAX);
+    WEE_CHECK_EVAL(str_value, str_expr);
+
     /* test translation */
     WEE_CHECK_EVAL("", "${translate:}");
     WEE_CHECK_EVAL("abcdef", "${translate:abcdef}");
 
+    /* test user variables */
+    WEE_CHECK_EVAL("", "${define:}");
+    WEE_CHECK_EVAL("", "${define:test}");
+    WEE_CHECK_EVAL("", "${define:test,value}");
+    WEE_CHECK_EVAL("value", "${define:test,value}${test}");
+    WEE_CHECK_EVAL("8", "${define:test,${calc:5+3}}${test}");
+    WEE_CHECK_EVAL("value", "${define:buffer,value}${buffer}");
+
     /* test option */
     hashtable_set (secure_hashtable_data, "sec_option", "sec_value");
     WEE_CHECK_EVAL("sec_value", "${sec.data.sec_option}");
@@ -776,7 +990,8 @@ TEST(CoreEval, EvalExpression)
     hashtable_set (pointers, "my_null_pointer", (const void *)0x0);
     hashtable_set (pointers, "my_buffer_pointer", gui_buffers);
     hashtable_set (pointers, "my_other_pointer", (const void *)0x1234abcd);
-    WEE_CHECK_EVAL("x", "x${buffer.number");
+    WEE_CHECK_EVAL("x", "x${buffer.numbe");
+    WEE_CHECK_EVAL("x1", "x${buffer.number");
     WEE_CHECK_EVAL("x${buffer.number}1",
                    "x\\${buffer.number}${buffer.number}");
     WEE_CHECK_EVAL("1", "${buffer.number}");
@@ -815,6 +1030,13 @@ TEST(CoreEval, EvalExpression)
     WEE_CHECK_EVAL(str_value, "${window.buffer.local_variables}");
     WEE_CHECK_EVAL("core", "${window.buffer.local_variables.plugin}");
     WEE_CHECK_EVAL("weechat", "${window.buffer.local_variables.name}");
+    WEE_CHECK_EVAL("name,plugin", "${window.buffer.local_variables.keys_sorted()}");
+    WEE_CHECK_EVAL("name:weechat,plugin:core", "${window.buffer.local_variables.keys_values_sorted()}");
+    WEE_CHECK_EVAL("", "${window.buffer.local_variables.nonexisting_func()}");
+    WEE_CHECK_EVAL("", "${window.buffer.local_variables.nonexisting_func(}");
+    WEE_CHECK_EVAL("", "${window.buffer.local_variables.nonexisting_func)}");
+    WEE_CHECK_EVAL("", "${window.buffer.local_variables.keys( )}");
+    WEE_CHECK_EVAL("", "${window.buffer.local_variables.()}");
     hashtable_remove_all (pointers);
 
     /* test with another prefix/suffix */
@@ -970,6 +1192,18 @@ TEST(CoreEval, EvalReplaceRegex)
     hashtable_set (options, "regex_replace", "${re:z}");
     WEE_CHECK_EVAL("", "abc");
 
+    /* use replace index: add number before each item */
+    hashtable_remove (pointers, "regex");
+    hashtable_set (options, "regex", "[^,]+");
+    hashtable_set (options, "regex_replace", "${re:repl_index}.${re:0}");
+    WEE_CHECK_EVAL("1.item1,2.item2,3.item3", "item1,item2,item3");
+
+    /* use replace index: replace each letter by its position */
+    hashtable_remove (pointers, "regex");
+    hashtable_set (options, "regex", ".");
+    hashtable_set (options, "regex_replace", "${re:repl_index}");
+    WEE_CHECK_EVAL("1234", "test");
+
     hashtable_free (pointers);
     hashtable_free (extra_vars);
     hashtable_free (options);
